
/* Initng, a 6next generatind sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#undef FILENAME
#define FILENAME "service_db.c"
#include "initng.h"

/* #define DEBUG*/
#define _GNU_SOURCE
#include <stdio.h>
#include <alloca.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <assert.h>
#include "process_db.h"
#include "service_db.h"
#include "string_tools.h"
#include "active_db.h"

static service_h **config_database = NULL;
static int next_service_to_alloc = 0;
static void service_db_remove_all(service_h * from_service);

/* change service_h pointer in database, maby on a realloc */
void service_db_change(service_h * from, service_h * to)
{
    int i = 0;

    assert(from);
    for (i = 0;
         i <= next_service_to_alloc && next_service_to_alloc <= MAX_SERVICE;
         i++)
        if (config_database[i] == from)
            config_database[i] = to;
}

/* add service to data structure, return index or -1 */
service_h *service_db_add(service_h * s)
{
    service_h *tmp_serv = 0;
    int i = 0;

    assert(s);
    assert(config_database);

    /* Check so we are uniq */
    if ((tmp_serv = service_db_find_by_name(s->name)))
    {
#ifdef DEBUG
        printf(" ** FAIL " FILENAME
               " service_db_add(%s): WARNING: DUPLICATE. Alredy added!\n",
               s->name);
#endif
        return (tmp_serv);
    }

    /* to make sure that the relly first post is zeroed */
    if (!next_service_to_alloc)
        config_database[0] = NULL;

    while (i <= next_service_to_alloc && next_service_to_alloc <= MAX_SERVICE)
    {
        if (!config_database[i])
        {

            config_database[i] = s;

            if (i == next_service_to_alloc)
            {
                next_service_to_alloc++;
                config_database[next_service_to_alloc] = 0;
            }
#ifdef DEBUG
            printf(" ** " FILENAME
                   " add_service(%s): added att entry config_database[%i].\n",
                   config_database[next_service_to_alloc - 1]->name,
                   next_service_to_alloc - 1);
#endif
            return (config_database[next_service_to_alloc - 1]);
        }
        i++;
    }

    return (NULL);
}

/* returns a service mallocated copy from s */
int service_db_copy(service_h ** new_serv, char *name, service_h * s)
{
    int i;

    assert(s);
    assert(name);
    /* allocate new service */
    *new_serv = (service_h *) calloc(1, sizeof(service_h));
    /* copy service structure */
    memcpy(*new_serv, s, sizeof(service_h));
    /* name has to diff */
    (*new_serv)->name = name;

    /* walk thru all data, and copy all strings. to avoid unwanted suprises. */
    for (i = 0; i < M_D_F; i++)
    {
        if ((*new_serv)->data[i].type != EMPTY && (*new_serv)->data[i].string)
        {
            (*new_serv)->data[i].string = strdup((*new_serv)->data[i].string);
        }
    }

#ifdef DEBUG
    printf(" ** " FILENAME " copy_service(%s);\n", s->name);
#endif

    return (TRUE);
}

/* returns a empty service mallocated whith default values */
int service_db_new(service_h ** d, char *name, e_type type)
{
    int i;

    assert(name);
#ifdef DEBUG
    printf(" ** " FILENAME " default_service();\n");
#endif
    *d = (service_h *) calloc(1, sizeof(service_h));
    if (!*d)
    {
        printf("OUT OF MEMORY default_service().\n");
        return (FALSE);
    }
    (*d)->name = name;
    (*d)->type = type;
    for (i = 0; i < M_D_F; i++)
    {
        (*d)->data[i].type = EMPTY;
        (*d)->data[i].string = NULL;
    }
    return (TRUE);
}

/* find service by name, returning pointer to service */
service_h *service_db_find_by_name(char *find)
{
    int i;

    assert(find);
    assert(config_database);

    for (i = 0; i < next_service_to_alloc; i++)
    {
        if (config_database[i])
        {
            if (st_service_cmp(find, config_database[i]->name))
            {
#ifdef DEBUG
                printf(" ** " FILENAME
                       " service_db_find_by_name(%s): Found config_database[%i].\n",
                       find, i);
#endif
                return (config_database[i]);
            }
        }
    }

    return (NULL);
}

/* free all the dynamic allocated strings in this service */
int service_db_free(service_h * to_free)
{
    assert(to_free);
#ifdef DEBUG
    printf(" ** " FILENAME " service_db_free(%s);\n", to_free->name);
#endif

    /* change other databases relations to this service to null */
    service_db_change(to_free, NULL);
    active_db_change_service_h(to_free, NULL);
    /* free all strings in service */
    service_db_remove_all(to_free);

    /* free service name */
    if (to_free->name)
        free(to_free->name);
    free(to_free);
    return (TRUE);
}

/* Walk thru every servise and clear it */
int service_db_init(void)
{
    /* allocate config database */
    config_database =
        (service_h **) calloc(MAX_SERVICE + 1, sizeof(service_h *));
    if (!config_database)
    {
        printf("Out of memory, can't allocate service database!\n");
        return (FALSE);
    }
#ifdef DEBUG
    printf(" ** " FILENAME " service_db_init();\n");
#endif
    return (TRUE);
}

/* Walk thru every servise and clear it */
void service_db_clear_all(void)
{
    int i;

    assert(config_database);
#ifdef DEBUG
    printf(" ** " FILENAME " service_db_clear();\n");
#endif
    for (i = 0; i <= next_service_to_alloc; i++)
    {
        if (config_database[i])
        {
            service_db_free(config_database[i]);
            config_database[i] = NULL;
        }
    }
}

/* Walk thru every servise and clear it */
void service_db_free_all(void)
{
    assert(config_database);
    service_db_clear_all();
    free(config_database);
    config_database = NULL;
}

char *service_db_get_string(e_sdt type, service_h * from_service)
{
    int i = 0;

    assert(from_service);
    if (type == STRING_NAME)
        return (from_service->name);

    /* search for type */
    for (; i < M_D_F; i++)
        if (from_service->data[i].type == type
            && from_service->data[i].string)
            return (from_service->data[i].string);

    /* did not got any match */
#ifdef DEBUG
    printf("String type %i dont exist for %s.\n", type, from_service->name);
#endif
    return (NULL);
}

char *service_db_get_next_string(e_sdt type, service_h * from_service,
                                 char *last)
{
    int i = 0;

    assert(from_service);
    if (type == STRING_NAME)
        return (from_service->name);

    /* fast forward */
    if (last != NULL)
        for (i = 0; i < M_D_F; i++)
            if (from_service->data[i].type == type &&
                from_service->data[i].string == last)
            {
                i++;
                break;
            }

    /* search for type */
    for (; i < M_D_F; i++)
        if (from_service->data[i].type == type
            && from_service->data[i].string)
            return (from_service->data[i].string);

    /* did not got any match */
#ifdef DEBUG
    printf("String type %i dont exist for %s.\n", type, from_service->name);
#endif
    return (NULL);
}

/* this is a funtion to add a string, that it can exist many of */
void
service_db_set_another_string(e_sdt type, service_h * from_service,
                              char *string)
{
    int i;

    assert(from_service);
    assert(string);
    if (type == STRING_NAME)
    {
        from_service->name = string;
        return;
    }
#ifdef DEBUG
    printf("data_set_another_string(%i,%s,\"%s\");\n", type,
           from_service->name, string);
#endif

    /* find a free spot */
    for (i = 0; i < M_D_F; i++)
        if (from_service->data[i].type == EMPTY)
        {
            /* add type to database */
            from_service->data[i].type = type;
            from_service->data[i].string = string;
            return;
        }
}

/* this have to be only one of */
void service_db_set_string(e_sdt type, service_h * from_service, char *string)
{
    int i;

    assert(from_service);
    assert(string);
    if (type == STRING_NAME)
    {
        from_service->name = string;
        return;
    }
#ifdef DEBUG
    printf("data_set_string(%i,%s,%s);\n", type, from_service->name, string);
#endif
    /* check if it is enabled allredy */
    for (i = 0; i < M_D_F; i++)
    {
        if (from_service->data[i].type == type)
        {
            if (from_service->data[i].string)
                free(from_service->data[i].string);
            from_service->data[i].string = string;
            return;
        }
    }

    /* find a free spot */
    for (i = 0; i < M_D_F; i++)
        if (from_service->data[i].type == EMPTY)
        {
            /* add type to database */
            from_service->data[i].type = type;
            from_service->data[i].string = string;
            return;
        }
}

/* Walk tru and count */
int service_db_count_type(e_sdt type, service_h * from_service)
{
    int i;
    int count = 0;

    assert(from_service);
    if (type <= 0)
    {
        for (i = 0; i < M_D_F; i++)
            if (from_service->data[i].type != EMPTY)
                count++;
        return count;
    }

    /* check if it is enabled allredy */
    for (i = 0; i < M_D_F; i++)
        if (from_service->data[i].type == type)
            count++;
    return count;
}

int service_db_is(e_sdt type, service_h * from_service)
{
    int i;

    assert(from_service);

    /* search for type */
    for (i = 0; i < M_D_F; i++)
    {
        if (from_service->data[i].type == type)
        {
            return (TRUE);
        }
    }
    /* did not got any match */
    return (FALSE);
}

void service_db_set(e_sdt type, service_h * from_service)
{
    int i;

    assert(from_service);

#ifdef DEBUG
    printf("data_set(%i,%s);\n", type, from_service->name);
#endif
    /* check if it is enabled allredy */
    for (i = 0; i < M_D_F; i++)
        if (from_service->data[i].type == type)
            return;

    /* add type to database */
    for (i = 0; i < M_D_F; i++)
        if (from_service->data[i].type == EMPTY)
        {
            from_service->data[i].type = type;
            from_service->data[i].string = NULL;
            return;
        }
}

void service_db_remove(e_sdt type, service_h * from_service)
{
    int i;

    assert(from_service);
    /* check if it is enabled allredy */
    for (i = 0; i < M_D_F; i++)
        if (from_service->data[i].type == type)
        {
            if (from_service->data[i].string)
                free(from_service->data[i].string);
            from_service->data[i].type = EMPTY;
        }
}

/* walk thru and kill all strings */
static void service_db_remove_all(service_h * from_service)
{
    int i;

    assert(from_service);
    /* check if it is enabled allredy */
    for (i = 0; i < M_D_F; i++)
    {
        if (from_service->data[i].string)
            free(from_service->data[i].string);
        from_service->data[i].type = EMPTY;
    }
}

/* P R I N T   S E R V I C E */

#ifdef PRINT_SERVICE

/* get position in database */
static int get_id(service_h * s)
{
    int i;

    assert(s);
    for (i = 0; i <= next_service_to_alloc; i++)
    {
        if (config_database[i])
            if (s == config_database[i])
            {
                return (i);
            }
    }
    return (-1);
}

static void service_db_pr(e_sdt type, char *value)
{
    char *dup=NULL;
    if (value) {
    
	if(strlen(value)>MAX_PRINT_LENGHT) {
	    int i=0;
	    dup=strndup(value,MAX_PRINT_LENGHT);
	    
	    /* replace enters with '$' */
	    for(i=0;i<=MAX_PRINT_LENGHT;i++) if(dup[i]=='\n') dup[i]='$';
    	    printf(" @@ %i:         \"%s...\"\n", type, dup);
	    free(dup);
	} else {
    	    printf(" @@ %i:         \"%s\"\n", type, value);
	}
    }
}

/* some stdout */
void service_db_print(service_h * s)
{
    int i;

    assert(s);
    assert(s->name);

    if (s->type == TYPE_SERVICE)
        printf("\n#%i   service \"%s\"\n", get_id(s), s->name);
    else if (s->type == TYPE_CLASS)
        printf("\n#%i   class \"%s\"\n", get_id(s), s->name);
    printf("    ------------------------ \n");
    for (i = 0; i < M_D_F; i++)
    {
        if (s->data[i].type != EMPTY)
        {
            service_db_pr(s->data[i].type, s->data[i].string);
        }
    }
}

/* Walk thru every servise and print it all */
void service_db_print_all(void)
{
    int i;

    assert(config_database);
#ifdef DEBUG
    printf("\n\n ** " FILENAME " print_all():\n");
#endif
    for (i = 0; i <= next_service_to_alloc; i++)
    {
        if (config_database[i])
            service_db_print(config_database[i]);
    }
}

#endif
